"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanupOutlookElementAttributes = exports.cleanupGMailElementAttributes = exports.amendGmailNodes = exports.amendOutlookNodes = exports.pruneOutlookElement = exports.pruneGmailElement = void 0;
const DUMMY_QR_CODE_ID = "dummyQrCode";
const elementsToRemove = { br: true, hr: true };
const attributesToKeep = {
    alt: true,
    src: true,
    cite: true,
    data: true,
    datetime: true,
    href: true,
    value: true,
};
const isDummyQrCode = (element) => {
    if (element.id === DUMMY_QR_CODE_ID) {
        return true;
    }
};
const pruneElement = (element) => {
    if (isDummyQrCode(element)) {
        return true;
    }
    return !!elementsToRemove[element.nodeName.toLowerCase()];
};
const pruneGmailElement = (element) => {
    return pruneElement(element);
};
exports.pruneGmailElement = pruneGmailElement;
/**
 * Returns true if element should be completely removed
 * @param element
 */
const pruneOutlookElement = (element) => {
    if (pruneElement(element)) {
        return true;
    }
    // Remove Outlook generic <o:*> tags
    return !!element.nodeName.toLowerCase().startsWith("o:");
};
exports.pruneOutlookElement = pruneOutlookElement;
const cloneAnchorFromPane = (a, pane) => {
    try {
        const url = new URL(a.href);
        // If this is external url
        if (url.host && url.protocol) {
            pane.parentNode.insertBefore(a.cloneNode(false), pane);
        }
    }
    catch (_a) {
        return;
    }
};
const amendOutlookNodes = (document) => {
    /**
     * Get rid of attachments panes
     */
    const attachmentsPanesConatiner = document.getElementById("OwaReferenceAttachments");
    const attachmentsPanesContainerEnd = document.getElementById("OwaReferenceAttachmentsEnd");
    if (attachmentsPanesConatiner) {
        const as = attachmentsPanesConatiner.getElementsByTagName("a");
        Array.from(as).forEach((a) => {
            cloneAnchorFromPane(a, attachmentsPanesConatiner);
        });
        attachmentsPanesConatiner.parentNode.removeChild(attachmentsPanesConatiner);
    }
    attachmentsPanesContainerEnd &&
        attachmentsPanesContainerEnd.parentNode.removeChild(attachmentsPanesContainerEnd);
    /**
     * Unwind spans, because sometimes Outlook wraps everything into span after sending
     */
    const spans = document.getElementsByTagName("span");
    /**
     * Sort spans by depth to start unwinding the deepest ones, which does not contain nested spans
     */
    const spansDepths = {};
    Array.from(spans).forEach((span) => {
        let descendant = span;
        let parent = descendant.parentNode;
        let depth = 0;
        while (parent && descendant !== parent) {
            descendant = parent;
            parent = descendant.parentNode;
            depth++;
        }
        if (!spansDepths[depth]) {
            spansDepths[depth] = [];
        }
        spansDepths[depth].push(span);
    });
    Object.keys(spansDepths)
        .sort((a, b) => parseInt(b) - parseInt(a))
        .forEach((depth) => {
        spansDepths[depth].forEach((span) => {
            let child = span.firstChild;
            const parent = span.parentNode;
            while (child) {
                parent.insertBefore(child.cloneNode(true), span);
                child = child.nextSibling;
            }
            span.parentNode.removeChild(span);
        });
    });
};
exports.amendOutlookNodes = amendOutlookNodes;
const amendGmailNodes = (document) => {
    /**
     * Look for attachments panes and remove everything but liks
     */
    const attachmentsPanes = Array.from(document.getElementsByClassName("gmail_chip"));
    attachmentsPanes.forEach((pane) => {
        const as = pane.querySelectorAll("a");
        as.forEach((a) => {
            cloneAnchorFromPane(a, pane);
        });
    });
    attachmentsPanes.forEach((pane) => {
        pane.parentNode.removeChild(pane);
    });
};
exports.amendGmailNodes = amendGmailNodes;
const cleanupGMailElementAttributes = (element) => {
    if (element.attributes.length > 0) {
        for (const attribute of element.attributes) {
            if (attribute.name === "data-surl") {
                element.setAttribute("src", attribute.value);
            }
        }
        for (let i = 0; i < element.attributes.length; i++) {
            const attribute = element.attributes[i];
            if (!attributesToKeep[attribute.name]) {
                element.removeAttribute(attribute.name);
                i--;
            }
        }
    }
};
exports.cleanupGMailElementAttributes = cleanupGMailElementAttributes;
const cleanupOutlookElementAttributes = (element) => {
    if (element.attributes.length > 0) {
        for (const attribute of element.attributes) {
            let valueSplit = attribute.value.split(" ");
            valueSplit = valueSplit.map((value) => value.startsWith("x_") ? value.replace("x_", "") : value);
            element.setAttribute(attribute.name, valueSplit.join(" "));
        }
        for (let i = 0; i < element.attributes.length; i++) {
            const attribute = element.attributes[i];
            if (!attributesToKeep[attribute.name]) {
                element.removeAttribute(attribute.name);
                i--;
            }
        }
    }
};
exports.cleanupOutlookElementAttributes = cleanupOutlookElementAttributes;
